#include <string.h>

#include <Application.h>
#include <Roster.h>
#include <Message.h>
#include <InterfaceDefs.h>

#include "EasyMoveFilter.h"

const bigtime_t HUGE_TIMEOUT = 2000000;

EasyMoveFilter::EasyMoveFilter() :
	win_msgr(0),
	init(B_OK)
{
	updateThread=spawn_thread(UpdateThreadFunc, "updateThread", B_NORMAL_PRIORITY, this);
	if(updateThread>=0)
		resume_thread(updateThread);
	
	oldWorkspace=current_workspace();
}

EasyMoveFilter::~EasyMoveFilter()
{
	if(win_msgr)
		delete win_msgr;
}

status_t EasyMoveFilter::InitCheck()
{
	return init;
}

filter_result EasyMoveFilter::Filter(BMessage *message, BList *outList)
{
	BPoint where;
	filter_result ret=B_DISPATCH_MESSAGE;
	
	int32 workspace=current_workspace();
	if(workspace!=oldWorkspace)
	{
		MoveWinToWorkspace(oldWorkspace, workspace);
		oldWorkspace=current_workspace();
	}
	
	switch(message->what)
	{
		case B_MOUSE_DOWN:
			if(message->FindInt32("buttons"  , &buttons  )!=B_OK) break;
			if(message->FindInt32("modifiers", &modifiers)!=B_OK) break;
			if(((modifiers & prefs.GetModifierMask())==prefs.GetModifierMask()) && (buttons & prefs.GetButtonMask()))
			{
				GetActiveWindow();
				if(win_msgr)
					ret=B_SKIP_MESSAGE;
			}
			break;
			
		case B_MOUSE_MOVED:
			if(message->FindPoint("where"  , &where  )!=B_OK) break;
			if(win_msgr)
				MoveWinBy(where-oldPos);
			oldPos=where;
			break;
			
		case B_MOUSE_UP:
			if(win_msgr)
			{
				delete win_msgr;
				win_msgr=0;
				ret=B_SKIP_MESSAGE;
			}
			break;
	};
	
	return ret;
}

void EasyMoveFilter::MoveWinToWorkspace(int32 oldWS, int32 newWS)
{
	if(!win_msgr)
		return;
	
	BMessage reply;
	int32 workspace;
	BMessage workspace_msg(B_GET_PROPERTY);
	workspace_msg.AddSpecifier("Workspaces");
	workspace_msg.AddSpecifier("Window", win_num);
	if(win_msgr->SendMessage(&workspace_msg, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT)!=B_OK)
		return;
	if(reply.FindInt32("result", 0, &workspace)!=B_OK)
		return;
	if(!(workspace & 1<<newWS))
	{
		workspace&=!(1<<oldWS);
		workspace|=  1<<newWS;
		BMessage workspace_msg2(B_SET_PROPERTY);
		workspace_msg2.AddSpecifier("Workspaces");
		workspace_msg2.AddSpecifier("Window", win_num);
		workspace_msg2.AddInt32("data", workspace);
		win_msgr->SendMessage(&workspace_msg2);
	}
}

void EasyMoveFilter::MoveWinBy(const BPoint &point)
{
	BMessage msg(B_WINDOW_MOVE_BY);
	msg.AddSpecifier("Window", win_num);
	msg.AddPoint("data", point);
	win_msgr->SendMessage(&msg);
}

void EasyMoveFilter::GetActiveWindow()
{
	if(win_msgr)
		delete win_msgr;
	win_msgr=0;
	
	app_info info;
	if(be_roster->GetActiveAppInfo(&info)!=B_OK)
		return;
	
	BMessenger msgr(0, info.team);
	BMessage reply;
	int32 wincount;
	
	BMessage count_msg(B_COUNT_PROPERTIES);
	count_msg.AddSpecifier("Window");
	if(msgr.SendMessage(&count_msg, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT)!=B_OK)
		return;
	if(reply.FindInt32("result", 0, &wincount)!=B_OK)
		return;
	for(int i=0; i<wincount; i++)
	{
		bool active;
		BMessage active_msg(B_GET_PROPERTY);
		active_msg.AddSpecifier("Active");
		active_msg.AddSpecifier("Window", i);
		if(msgr.SendMessage(&active_msg, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT)!=B_OK)
			continue;
		if(reply.FindBool("result", 0, &active)!=B_OK)
			continue;
		if(active)
		{
			const char *title;
			BMessage wintitle_msg(B_GET_PROPERTY);
			wintitle_msg.AddSpecifier("Title");
			wintitle_msg.AddSpecifier("Window", i);
			if(msgr.SendMessage(&wintitle_msg, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT)!=B_OK)
				return;
			if(reply.FindString("result", 0, &title)!=B_OK)
				return;
			if(strcmp(info.signature, "application/x-vnd.Be-TSKB") || strcmp(title, "Deskbar"))
			{
				if(!(strcmp(info.signature, "application/x-vnd.Be-TRAK") || strcmp(title, "Desktop")))
				{
					BRect frame;
					BMessage frame_msg(B_GET_PROPERTY);
					frame_msg.AddSpecifier("Frame");
					frame_msg.AddSpecifier("Window", i);
					if(msgr.SendMessage(&frame_msg, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT)!=B_OK)
						return;
					if(reply.FindRect("result", 0, &frame)!=B_OK)
						return;
					if(!frame.left && !frame.top)
						return;
				}
				win_msgr=new BMessenger(msgr);
				win_num=i;
			}
		}
	}
}

status_t EasyMoveFilter::UpdateThreadFunc(void *cookie)
{
	port_id updatePrefsPort=create_port(1, "EasyMove UpdatePrefsPort");
	if(updatePrefsPort<0)
		return updatePrefsPort;
	
	while(1)
	{
		int32 msg_code;
		int8 buff[10];
		status_t err=read_port(updatePrefsPort, &msg_code, buff, 10);
		if(err<0)
			return err;
		((EasyMoveFilter *)cookie)->prefs.Update();
	}
	
	return B_OK;
}
