#include <iostream>
#include <string.h>

#include <storage/Path.h>
#include <storage/File.h>
#include <storage/Directory.h>
#include <storage/FindDirectory.h>
#include <interface/View.h>
#include <interface/InterfaceDefs.h>

#include "Preferences.h"

Preferences::Preferences()
{
	Load();
}

Preferences::~Preferences()
{
	Save();
}

void Preferences::Update()
{
	Load();
}

uint32 Preferences::GetModifierMask()
{
	return mModifierMask;
}

void Preferences::SetModifierMask(uint32 mask)
{
	mModifierMask=mask;
}

void Preferences::SetModifierMaskToDefault()
{
	mModifierMask=B_COMMAND_KEY;
}

uint32 Preferences::GetButtonMask()
{
	return mButtonMask;
}

void Preferences::SetButtonMask(uint32 mask)
{
	mButtonMask=mask;
}

void Preferences::SetButtonMaskToDefault()
{
	mButtonMask=B_PRIMARY_MOUSE_BUTTON | B_SECONDARY_MOUSE_BUTTON | B_TERTIARY_MOUSE_BUTTON;
}

void Preferences::SetToDefault()
{
	SetModifierMaskToDefault();
	SetButtonMaskToDefault();
}

BFile *Preferences::OpenFile(uint32 openMode)
{
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	BDirectory dir(path.Path());
	BFile *file=new BFile(&dir, PrefsFilename, openMode);
	status_t err;
	err=file->InitCheck();
	if(err!=B_OK)
	{
		if(openMode&B_CREATE_FILE)
			cerr <<"could not open file: " <<strerror(err) <<endl;
		delete file;
		return 0;
	}
	return file;
}

void Preferences::Load()
{
	SetToDefault();
	BFile *file=OpenFile(B_READ_ONLY);
	if(!file)
		return;
	file->Read(&mModifierMask, 4);
	file->Read(&mButtonMask, 4);
	delete file;
}

void Preferences::Save()
{
	BFile *file=OpenFile(B_WRITE_ONLY | B_CREATE_FILE);
	if(!file)
		return;
	
	file->Write(&mModifierMask, 4);
	file->Write(&mButtonMask, 4);
	
	delete file;
}
