#include <Application.h>
#include <TextView.h>
#include <View.h>

#include "ModifierView.h"
#include "SettingsWin.h"

const SettingsWin::MaskListType SettingsWin::ModifierList[]=
{
	{B_SHIFT_KEY,	"Shift"},
	{B_COMMAND_KEY,	"Command"},
	{B_CONTROL_KEY,	"Control"},
	{B_OPTION_KEY,	"Option"},
	{B_MENU_KEY,	"Menu"}
};

const int32 SettingsWin::NbModifiers=5;

const SettingsWin::MaskListType SettingsWin::ButtonList[]=
{
	{B_PRIMARY_MOUSE_BUTTON,	"Primary mouse button"},
	{B_SECONDARY_MOUSE_BUTTON,	"Secondary mouse button"},
	{B_TERTIARY_MOUSE_BUTTON,	"Tertiary mouse button"}
};

const int32 SettingsWin::NbButtons=3;

SettingsWin::SettingsWin() :
	BWindow(BRect(100, 100, 400, 300), "EasyMove Settings", B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS)
{
	bg=new BView(BRect(Bounds()), "", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	bg->SetViewColor(216, 216, 216);
	AddChild(bg);
	
	BTextView *textView=new BTextView(BRect(10, 10, Bounds().right-10, 50), "", BRect(10, 10, Bounds().right-10, 50), B_FOLLOW_NONE, B_WILL_DRAW);
	textView->MakeEditable(false);
	textView->MakeSelectable(false);
	textView->SetWordWrap(false);
	textView->SetViewColor(216, 216, 216);
	textView->SetText("EasyMove will work only if all the modifiers are present,\nand one of the buttons pushed");
	textView->ResizeToPreferred();
	bg->AddChild(textView);
	
	mModifierViewList=new ModifierView*[NbModifiers];
	BRect frame(10, 60, 10, 60);
	
	for(int32 i=0; i<NbModifiers; i++)
	{
		mModifierViewList[i]=new ModifierView(frame,
											  ModifierList[i].mask,
											  ModifierList[i].label,
											  ModifierList[i].mask & mPrefs.GetModifierMask(),
											  new BMessage(MODIFIER_CHANGED));
		mModifierViewList[i]->ResizeToPreferred();
		bg->AddChild(mModifierViewList[i]);
		frame.OffsetTo(frame.left, mModifierViewList[i]->Frame().bottom+5);
	}
	
	mButtonViewList=new ModifierView*[NbButtons];
	frame=BRect(150, 60, 150, 60);
	
	for(int32 i=0; i<NbButtons; i++)
	{
		mButtonViewList[i]=new ModifierView(frame,
										    ButtonList[i].mask,
										    ButtonList[i].label,
										    ButtonList[i].mask & mPrefs.GetButtonMask(),
										    new BMessage(BUTTON_CHANGED));
		mButtonViewList[i]->ResizeToPreferred();
		bg->AddChild(mButtonViewList[i]);
		frame.OffsetTo(frame.left, mButtonViewList[i]->Frame().bottom+5);
	}
}

SettingsWin::~SettingsWin()
{
	delete mModifierViewList;
}

void SettingsWin::MessageReceived(BMessage *message)
{
	uint32 mask=0;
	
	switch(message->what)
	{
		case MODIFIER_CHANGED:
			for(int32 i=0; i<NbModifiers; i++)
				mask|=mModifierViewList[i]->GetMask();
			if(mask)
				mPrefs.SetModifierMask(mask);
			else
			{
				mPrefs.SetModifierMaskToDefault();
				for(int32 i=0; i<NbModifiers; i++)
					mModifierViewList[i]->SetValue(ModifierList[i].mask & mPrefs.GetModifierMask());
			}
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case BUTTON_CHANGED:
			for(int32 i=0; i<NbButtons; i++)
				mask|=mButtonViewList[i]->GetMask();
			if(mask)
				mPrefs.SetButtonMask(mask);
			else
			{
				mPrefs.SetButtonMaskToDefault();
				for(int32 i=0; i<NbButtons; i++)
					mButtonViewList[i]->SetValue(ButtonList[i].mask & mPrefs.GetButtonMask());
			}
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		default:
			BWindow::MessageReceived(message);
	}
}

bool SettingsWin::QuitRequested()
{
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
	return true;
}

void SettingsWin::SendUpdateMessage()
{
	port_id port=find_port("EasyMove UpdatePrefsPort");
	if(port<0)
		return;
	write_port(port, 0, 0, 0);
}
